import importlib
import jax.numpy as jnp

import exogibbs.api.equilibrium as eqmod
from exogibbs.api.equilibrium import EquilibriumOptions


class FakeSetup:
    """Minimal stand-in for ChemicalSetup for vectorized API testing."""

    def __init__(self, A):
        self.formula_matrix = A

    def hvector_func(self, T):
        K = self.formula_matrix.shape[1]
        return jnp.zeros((K,))


def test_equilibrium_profile_shapes_and_values(monkeypatch):
    """Simple happy-path test for equilibrium_profile with a stubbed minimizer.

    Ensures batched shapes are correct and values match the stub behavior
    (ln_n = 0 => n = 1, x uniform, ntot = K) across all layers.
    """
    E, K, N = 2, 3, 4
    A = jnp.array([[1, 0, 1], [0, 1, 0]], dtype=jnp.float64)
    setup = FakeSetup(A)

    def stub_minimize_gibbs(state, ln_nk0, ln_ntot0, A_in, hfunc, **kwargs):
        # Basic sanity checks on inputs
        assert A_in.shape == A.shape
        assert ln_nk0.shape == (K,)
        assert ln_ntot0.shape == ()
        return jnp.zeros((K,), dtype=jnp.result_type(ln_nk0, A_in.dtype))

    monkeypatch.setattr(
        "exogibbs.api.equilibrium.minimize_gibbs", stub_minimize_gibbs, raising=True
    )

    # Profile inputs (N layers)
    T = jnp.linspace(1000.0, 2000.0, N)
    P = jnp.linspace(0.1, 1.0, N)
    b = jnp.array([1.0, 2.0], dtype=jnp.float64)

    out = eqmod.equilibrium_profile(
        setup, T, P, b, options=EquilibriumOptions(epsilon_crit=1e-11, max_iter=50)
    )

    # Batched shapes
    assert out.ln_n.shape == (N, K)
    assert out.n.shape == (N, K)
    assert out.x.shape == (N, K)
    assert out.ntot.shape == (N,)

    # Stub behavior reflected across layers
    assert jnp.allclose(out.ln_n, 0.0)
    assert jnp.allclose(out.n, 1.0)
    assert jnp.allclose(out.x, jnp.ones((N, K)) / K)
    assert jnp.allclose(out.ntot, K)
    assert jnp.allclose(jnp.sum(out.x, axis=1), 1.0)




"""
Validation of Gibbs Minimization Against ykawashima's B4 code
============================================================

This example demonstrates and validates the ExoGibbs thermochemical equilibrium
solver against the code by ykawashima when she was at B4.

Updated to use the high-level API: exogibbs.api.equilibrium.equilibrium.
"""
from exogibbs.presets.ykb4 import prepare_ykb4_setup
import jax.numpy as jnp
from exogibbs.api.chemistry import ThermoState
from exogibbs.optimize.minimize import minimize_gibbs_core

from exogibbs.optimize.core import compute_ln_normalized_pressure

from jax import config

config.update("jax_enable_x64", True)

#chemical setup
chem = prepare_ykb4_setup()

# Thermodynamic conditions
temperature = 500.0  # K
P = 10.0  # bar
Pref = 1.0  # bar, reference pressure
ln_normalized_pressure = compute_ln_normalized_pressure(P, Pref)

# Initial guess for log number densities
ln_nk = jnp.zeros(chem.formula_matrix.shape[1])  # log(n_species)
ln_ntot = 0.0  # log(total number density)

# old reference elemental abundance b from yk's sample number densities
b_old_ref = jnp.array(
    [
        4.8774824e-04,
        1.6749767e00,
        1.6143440e-01,
        2.5438149e-07,
        1.3435642e-04,
        3.9624806e-06,
        9.7356725e-04,
        5.7690579e-07,
        3.0653933e-05,
        1.6687756e-07,
        1.9870969e-08,
        0.0000000e00,
    ]
)
# ThermoState instance
thermo_state = ThermoState(
    temperature=temperature,
    ln_normalized_pressure=ln_normalized_pressure,
    b_element_vector=b_old_ref,
)

# Convergence criteria
epsilon_crit = 1e-11
max_iter = 1000

ln_nk_result, _, icount = minimize_gibbs_core(
    thermo_state,
    ln_nk,
    ln_ntot,
    chem.formula_matrix,
    chem.hvector_func,
    epsilon_crit=epsilon_crit,
    max_iter=max_iter,
)
nk_result = jnp.exp(ln_nk_result)
print("icount", icount) 
#702 for clfac = 0.1, 347 for _cea_lambda


"""
Validation of Gibbs Minimization for Layered Systems 
===================================================================================

This example demonstrates and validates the ExoGibbs thermochemical equilibrium for the layered system

Updated to use the high-level API: exogibbs.api.equilibrium.equilibrium.
"""

from exogibbs.presets.ykb4 import prepare_ykb4_setup
from exogibbs.api.equilibrium import equilibrium_profile, EquilibriumOptions
import numpy as np
import jax.numpy as jnp
from jax import config

config.update("jax_enable_x64", True)


# Thermodynamic conditions
Pref = 1.0  # bar, reference pressure
Parr = jnp.logspace(-8, 2, 10)/Pref  # bar
Tarr = 1500*Parr**0.1


#chemical setup
chem = prepare_ykb4_setup()

##############################################################################
# Solve equilibrium via high-level API
# ------------------------------------
opts = EquilibriumOptions(epsilon_crit=1e-11, max_iter=1000)

res = equilibrium_profile(
    chem,
    Tarr,
    Parr,
    chem.b_element_vector_reference,
    Pref=Pref,
    options=opts,
)

##############################################################################
nk_result = res.n
print(nk_result)
